/* SPDX-License-Identifier: GPL-2.0 */
/*
 * ina231.h
 *
 * ina231 driver
 *
 * Copyright (c) 2020-2020 Huawei Technologies Co., Ltd.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 */

#ifndef _INA231_H_
#define _INA231_H_

#include <linux/init.h>
#include <linux/module.h>
#include <linux/device.h>
#include <linux/slab.h>
#include <linux/i2c.h>
#include <linux/gpio.h>
#include <linux/of.h>
#include <linux/of_device.h>
#include <linux/of_address.h>
#include <linux/of_gpio.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <linux/notifier.h>
#include <linux/raid/pq.h>
#include <linux/spinlock.h>
#include <linux/bitops.h>
#include <chipset_common/hwpower/direct_charge/direct_charger.h>

/* CONFIG reg=0x00, default=0x4127, RW */
#define INA231_CONFIG_REG                0x00

#define INA231_CONFIG_RST_MASK           BIT(15) /* reset bit */
#define INA231_CONFIG_RST_SHIFT          15
/* averaging mode */
#define INA231_CONFIG_AVG_MASK           (BIT(9) | BIT(10) | BIT(11))
#define INA231_CONFIG_AVG_SHIFT          9
/* bus voltage conversion time */
#define INA231_CONFIG_VBUSCT_MASK        (BIT(6) | BIT(7) | BIT(8))
#define INA231_CONFIG_VBUSCT_SHIFT       6
/* shunt voltage conversion time */
#define INA231_CONFIG_VSHUNTCT_MASK      (BIT(3) | BIT(4) | BIT(5))
#define INA231_CONFIG_VSHUNTCT_SHIFT     3
/* operating mode */
#define INA231_CONFIG_MODE_MASK          (BIT(0) | BIT(1) | BIT(2))
#define INA231_CONFIG_MODE_SHIFT         0

#define INA231_CONFIG_RST_ENABLE         1

#define INA231_CONFIG_AVG_NUM_1          0x0
#define INA231_CONFIG_AVG_NUM_4          0x1
#define INA231_CONFIG_AVG_NUM_16         0x2
#define INA231_CONFIG_AVG_NUM_64         0x3
#define INA231_CONFIG_AVG_NUM_128        0x4
#define INA231_CONFIG_AVG_NUM_256        0x5
#define INA231_CONFIG_AVG_NUM_512        0x6
#define INA231_CONFIG_AVG_NUM_1024       0x7

#define INA231_CONFIG_CT_140US           0x0
#define INA231_CONFIG_CT_204US           0x1
#define INA231_CONFIG_CT_332US           0x2
#define INA231_CONFIG_CT_588US           0x3
#define INA231_CONFIG_CT_1100US          0x4
#define INA231_CONFIG_CT_2116US          0x5
#define INA231_CONFIG_CT_4156US          0x6
#define INA231_CONFIG_CT_8244US          0x7

#define INA231_CONFIG_MODE_PD0           0x0 /* power-down */
#define INA231_CONFIG_MODE_SV_TRIG       0x1 /* shunt voltage, triggered */
#define INA231_CONFIG_MODE_BV_TRIG       0x2 /* bus voltage, triggered */
#define INA231_CONFIG_MODE_SB_TRIG       0x3 /* shunt and bus, triggered */
#define INA231_CONFIG_MODE_PD1           0x4 /* power-down */
#define INA231_CONFIG_MODE_SV_CONS       0x5 /* shunt voltage, continuous */
#define INA231_CONFIG_MODE_BV_CONS       0x6 /* bus voltage, continuous */
#define INA231_CONFIG_MODE_SB_CONS       0x7 /* shunt and bus, continuous */

/* SHUNT_VOLTAGE reg=0x01, default=0x0000, R */
#define INA231_SHUNT_VOLTAGE_REG         0x01

/* BUS_VOLTAGE reg=0x02, default=0x0000, R */
#define INA231_BUS_VOLTAGE_REG           0x02

#define INA231_BUS_VOLTAGE_STEP          125 /* 25mv */

/* POWER reg=0x03, default=0x0000, R */
#define INA231_POWER_REG                 0x03

/* CURRENT reg=0x04, default=0x0000, R */
#define INA231_CURRENT_REG               0x04

/* CALIBRATION reg=0x05, default=0x0000, RW */
#define INA231_CALIBRATION_REG           0x05

/* MASK_ENABLE reg=0x06, default=0x0000, RW */
#define INA231_MASK_ENABLE_REG           0x06

/* shunt voltage over-voltage */
#define INA231_MASK_ENABLE_SOL_MASK      BIT(15)
#define INA231_MASK_ENABLE_SOL_SHIFT     15
/* shunt voltage under-voltage */
#define INA231_MASK_ENABLE_SUL_MASK      BIT(14)
#define INA231_MASK_ENABLE_SUL_SHIFT     14
/* bus voltage over-voltage */
#define INA231_MASK_ENABLE_BOL_MASK      BIT(13)
#define INA231_MASK_ENABLE_BOL_SHIFT     13
/* bus voltage under-voltage */
#define INA231_MASK_ENABLE_BUL_MASK      BIT(12)
#define INA231_MASK_ENABLE_BUL_SHIFT     12
/* over-limit power */
#define INA231_MASK_ENABLE_POL_MASK      BIT(11)
#define INA231_MASK_ENABLE_POL_SHIFT     11
/* conversion ready */
#define INA231_MASK_ENABLE_CNVR_MASK     BIT(10)
#define INA231_MASK_ENABLE_CNVR_SHIFT    10
/* alert function flag */
#define INA231_MASK_ENABLE_AFF_MASK      BIT(4)
#define INA231_MASK_ENABLE_AFF_SHIFT     4
/* conversion ready flag */
#define INA231_MASK_ENABLE_CVRF_MASK     BIT(3)
#define INA231_MASK_ENABLE_CVRF_SHIFT    3
/* math overflow flag */
#define INA231_MASK_ENABLE_OVF_MASK      BIT(2)
#define INA231_MASK_ENABLE_OVF_SHIFT     2
/* alert polarity */
#define INA231_MASK_ENABLE_APOL_MASK     BIT(1)
#define INA231_MASK_ENABLE_APOL_SHIFT    1
/* alert altch enable */
#define INA231_MASK_ENABLE_LEN_MASK      BIT(0)
#define INA231_MASK_ENABLE_LEN_SHIFT     0

/* ALERT_LIMIT reg=0x07, default=0x0000, RW */
#define INA231_ALERT_LIMIT_REG           0x07

/* register count */
#define INA231_MAX_REGS                  8

#define INA231_NV_TO_MV                  1000000

#define INA231_DEVICE_DEFAULT_TEMP       25

#define INA231_INIT_FINISH               1
#define INA231_NOT_INIT                  0
#define INA231_ENABLE_INT_NOTIFY         1
#define INA231_DISABLE_INT_NOTIFY        0

#define INA231_INT_UNDER_CURRENT_MA      200
#define INA231_DEFAULT_RSHUNT_MOHM       10
#define INA231_CALL_DIVISOR              5120000

struct ina231_config_data {
	u16 config_sleep_in;
	u16 config_reset;
	u16 config_work;
	u16 calibrate_content;
	u16 mask_enable_content;
	u16 alert_limit_content;
	int shunt_voltage_lsb; /* nV/bit */
	int shunt_voltage_max;
	int bus_voltage_lsb; /* uV/bit */
	int bus_voltage_max;
	int current_lsb; /* uA/bit */
	int current_max;
	int power_lsb; /* uW/bit */
	int power_max;
};

struct ina231_device_info {
	struct device *dev;
	struct i2c_client *client;
	struct work_struct irq_work;
	struct nty_data nty_data;
	struct notifier_block nb;
	unsigned int notifier_type;
	spinlock_t int_lock;
	int gpio_int;
	int irq_int;
	bool is_int_en;
	int chip_already_init;
	unsigned int rshunt;
	int count;
	struct ina231_config_data *config;
	u32 ic_role;
	int init_finish_flag;
	int int_notify_enable_flag;
};

#endif /* _INA231_H_ */
