// SPDX-License-Identifier: GPL-2.0+
/*
 * bq2560x_charger.c
 *
 * bq2560x driver
 *
 * Copyright (c) 2012-2020 Huawei Technologies Co., Ltd.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/device.h>
#include <linux/delay.h>
#include <linux/jiffies.h>
#include <linux/platform_device.h>
#include <linux/slab.h>
#include <linux/i2c.h>
#include <linux/pm_wakeup.h>
#include <linux/usb/otg.h>
#include <linux/io.h>
#include <linux/gpio.h>
#include <linux/power_supply.h>
#include <linux/of.h>
#include <linux/of_device.h>
#include <linux/of_address.h>
#include <linux/of_gpio.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <linux/notifier.h>
#include <linux/mutex.h>
#include <linux/raid/pq.h>
#include <securec.h>
#include <chipset_common/hwpower/common_module/power_i2c.h>
#include <platform_include/cee/linux/adc.h>
#include <huawei_platform/log/hw_log.h>
#ifdef CONFIG_HUAWEI_HW_DEV_DCT
#include <huawei_platform/devdetect/hw_dev_dec.h>
#endif
#include <huawei_platform/power/huawei_charger.h>
#include <chipset_common/hwpower/hardware_monitor/uscp.h>
#include "bq2560x_charger.h"
#include <chipset_common/hwpower/battery/battery_temp.h>
#include <chipset_common/hwpower/common_module/power_algorithm.h>
#include <chipset_common/hwpower/common_module/power_dsm.h>
#include <chipset_common/hwpower/common_module/power_event_ne.h>
#include <chipset_common/hwpower/common_module/power_time.h>
#include <huawei_platform/hwpower/common_module/power_platform.h>
#include <chipset_common/hwpower/common_module/power_common_macro.h>
#include <chipset_common/hwpower/common_module/power_delay.h>
#include <chipset_common/hwpower/direct_charge/direct_charge_ic.h>
#include <chipset_common/hwpower/direct_charge/direct_charge_error_handle.h>
#include <huawei_platform/hwpower/common_module/power_platform.h>
#include <huawei_platform/hwpower/common_module/power_platform_macro.h>
#include <chipset_common/hwpower/common_module/power_common_macro.h>
#include <chipset_common/hwpower/direct_charge/direct_charge_device_id.h>
#include <chipset_common/hwpower/common_module/power_supply.h>
#include <chipset_common/hwpower/common_module/power_supply_interface.h>
#include <chipset_common/hwpower/direct_charge/direct_charge_adapter.h>
#include <chipset_common/hwpower/direct_charge/direct_charge_ic_manager.h>
#include <chipset_common/hwpower/charger/charger_common_interface.h>

#ifdef HWLOG_TAG
#undef HWLOG_TAG
#endif

#define HWLOG_TAG bq2560x_charger
HWLOG_REGIST();

static struct bq2560x_device_info *g_bq2560x_dev;

static bool g_hiz_mode = FALSE;
static int hiz_iin_limit_flag = HIZ_IIN_FLAG_FALSE;
static int g_cv_flag;
static int g_cv_policy;
static int g_bq2560x_cv;
static int g_bq2560x_ovp;
static bool g_shutdown_flag;
static int g_safe_time_cnt;
static time64_t g_safe_pre_time = SAFE_TIME_RESET;
static time64_t g_boot_time;
static int g_sgm41511_exist;

#define TERM_CURR                    840
#define LIMIT_CURR                   840

#define BUF_LEN                      26

#define GPIO_LOW                      0
#define GPIO_HIGH                     1
#define DELAY_10MS                    10
#define DELAY_50MS                    50
#define DELAY_100MS                   100
#define DELAY_1500MS                  1500
#define RETRY_MAX3                    3
#define MIN_CV                        4350
#define RETRY_TIMES                   3
#define DEFAULT_FAKE_DEVICE_TEMP      30
#define DEFAULT_SC_MAX_INPUT_CURRENT  2200
#define DEFAULT_SC_MAX_CHARGE_CURRENT 3000
#define MONITOR_WORK_DELAY_TIME       5000
#define MONITOR_WORK_DEBOUNCE_TIME    500

static int bq2560x_fcp_reset(void *dev_data);

static int bq2560x_set_ovpthreshold(int threshld);

static int bq2560x_write_byte(u8 reg, u8 value)
{
	struct bq2560x_device_info *di = g_bq2560x_dev;

	if (!di) {
		hwlog_err("chip not init\n");
		return -EIO;
	}

	return power_i2c_u8_write_byte(di->client, reg, value);
}

static int bq2560x_read_byte(u8 reg, u8 *value)
{
	struct bq2560x_device_info *di = g_bq2560x_dev;

	if (!di) {
		hwlog_err("chip not init\n");
		return -EIO;
	}

	return power_i2c_u8_read_byte(di->client, reg, value);
}

static int bq2560x_write_mask(u8 reg, u8 mask, u8 shift, u8 value)
{
	int ret;
	u8 val = 0;

	if (!g_bq2560x_dev) {
		hwlog_err("chip not init\n");
		return -EIO;
	}

	mutex_lock(&g_bq2560x_dev->io_lock);
	ret = bq2560x_read_byte(reg, &val);
	if (ret < 0) {
		mutex_unlock(&g_bq2560x_dev->io_lock);
		return ret;
	}

	val &= ~mask;
	val |= ((value << shift) & mask);

	ret = bq2560x_write_byte(reg, val);
	mutex_unlock(&g_bq2560x_dev->io_lock);
	return ret;
}

static int bq2560x_read_mask(u8 reg, u8 mask, u8 shift, u8 *value)
{
	int ret;
	u8 val = 0;

	ret = bq2560x_read_byte(reg, &val);
	if (ret < 0)
		return ret;

	val &= mask;
	val >>= shift;
	*value = val;

	return 0;
}

static int bq2560x_device_check(void)
{
	u8 reg = 0;
	u8 pn_value;
	int ret;

	if (!g_bq2560x_dev) {
		hwlog_err("%s chip not init\n", __func__);
		return -EIO;
	}

	ret = bq2560x_read_byte(BQ2560X_REG_VPRS, &reg);
	if (ret)
		return CHARGE_IC_BAD;

	hwlog_info("device_check [%x]=0x%x\n", BQ2560X_REG_VPRS, reg);

	pn_value = (reg & BQ2560X_REG_VPRS_PN_MASK) >> BQ2560X_REG_VPRS_PN_SHIFT;
	g_bq2560x_dev->vendor_id = pn_value;

	switch (pn_value) {
	case BQ2560X_VENDOR_ID:
		hwlog_info("bq2560x is good\n");
		return CHARGE_IC_GOOD;
	case SGM41512H_VENDOR_ID:
		hwlog_info("sgm41512H is good\n");
		return CHARGE_IC_GOOD;
	case SY6974B_VENDOR_ID:
		hwlog_info("sy6974b is good\n");
		return CHARGE_IC_GOOD;
	case ETA6963_VENDOR_ID:
		hwlog_info("eta6963 is good\n");
		return CHARGE_IC_GOOD;
	case AH0072_VENDOR_ID:
		hwlog_info("sgm41513d is good\n");
		return CHARGE_IC_GOOD;
	case SC18601_VENDOR_ID:
		hwlog_info("sc18601 is good\n");
		return CHARGE_IC_GOOD;
	default:
		hwlog_err("no chip match, return ic bad\n");
		return CHARGE_IC_BAD;
	}
}

static int sgm41511_device_check(void)
{
	u8 reg = 0;
	int ret;

	ret = bq2560x_read_byte(BQ2560X_REG_VPRS, &reg);
	if (ret)
		return 0;

	hwlog_info("device_check %x = 0x%x\n", BQ2560X_REG_VPRS, reg);

	if (((reg & BQ2560X_REG_VPRS_PART_MASK) >>
		BQ2560X_REG_VPRS_PART_SHIFT) == SGM41511H_VENDOR_ID) {
		hwlog_info("device exists\n");
		return 1;
	}

	hwlog_err("device not exists\n");
	return 0;
}

static int bq2560x_5v_chip_init(struct bq2560x_device_info *di)
{
	int ret;

	g_hiz_mode = FALSE;

	if (!di) {
		hwlog_err("chip not init\n");
		return -EIO;
	}

	/* boost mode current limit = 1000mA */
	ret = bq2560x_write_byte(BQ2560X_REG_CCC, 0xa1);
	if (ret < 0)
		hwlog_err("init otg cur fail\n");

	/* I2C watchdog timer setting = 80s */
	/* fast charge timer setting = 12h */
	ret = bq2560x_write_byte(BQ2560X_REG_CTTC, 0xAF);
	if (ret < 0)
		hwlog_err("init wdt time fail\n");

	/* iprechg = 256ma,iterm current = 128ma */
	if (di->vendor_id == AH0072_VENDOR_ID)
		ret = bq2560x_write_byte(BQ2560X_REG_PCTCC, 0xFA);
	else
		ret = bq2560x_write_byte(BQ2560X_REG_PCTCC, 0x42);
	if (ret < 0)
		hwlog_err("init pre_chg_cur fail\n");

	ret = bq2560x_write_mask(BQ2560X_REG_MOC,
		BQ2560X_REG_MOC_VDPM_BAT_TRACK_MASK,
		BQ2560X_REG_MOC_VDPM_BAT_TRACK_SHIFT,
		REG07_VDPM_BAT_TRACK_DISABLE);
	if (ret < 0)
		hwlog_err("disable auto vdpm fail\n");

	ret = bq2560x_set_ovpthreshold(g_bq2560x_ovp);
	if (ret < 0)
		hwlog_err("init acov threshold fail\n");

	hiz_iin_limit_flag = HIZ_IIN_FLAG_FALSE;

	/* enable charging */
	gpio_set_value(di->gpio_cd, 0);

	return ret;
}

static int bq2560x_chip_init(struct charge_init_data *init_crit)
{
	int ret = -1;
	struct bq2560x_device_info *di = g_bq2560x_dev;

	if (!di || !init_crit) {
		hwlog_err("di or init_crit is null\n");
		return -ENOMEM;
	}

	if (g_sgm41511_exist) {
		g_safe_time_cnt = 0;
		g_safe_pre_time = SAFE_TIME_RESET;
		g_boot_time = power_get_monotonic_boottime();
	}

	di->vbus = init_crit->vbus;
	switch (init_crit->vbus) {
	case ADAPTER_5V:
		/* reset fcp before chip init with vbus 5v */
		if (di->fcp_support)
			(void)bq2560x_fcp_reset(di);

		ret = bq2560x_5v_chip_init(di);
		break;
	default:
		hwlog_err("invaid init_crit vbus mode\n");
		break;
	}

	return ret;
}

static int bq2560x_set_input_current(int value)
{
	int val;

	/*
	 * IC precision deviation by hw: +100mA(current >= 1700)
	 * max value protect: 3200mA
	 */
	if (value >= AC_IIN_CURRENT_THRESHOLD) {
		value += AC_IIN_CURRENT_OFFSET;
		if (value > AC_IIN_MAX_CURRENT)
			value = AC_IIN_MAX_CURRENT;
	}

	val = (value - REG00_IINLIM_BASE) / REG00_IINLIM_LSB;

	hwlog_info("set_input_current [%x]=0x%x\n", BQ2560X_REG_ISC, val);

	return bq2560x_write_mask(BQ2560X_REG_ISC,
		BQ2560X_REG_ISC_IINLIM_MASK,
		BQ2560X_REG_ISC_IINLIM_SHIFT,
		val);
}

static int bq2560x_get_input_current_set(void)
{
	u8 reg = 0;
	int value;
 
	(void)bq2560x_read_mask(BQ2560X_REG_ISC, BQ2560X_REG_ISC_IINLIM_MASK,
		BQ2560X_REG_ISC_IINLIM_SHIFT, &reg);
	value = (int)reg * REG00_IINLIM_LSB + REG00_IINLIM_BASE;
	if (value > AC_IIN_CURRENT_THRESHOLD)
		value -= AC_IIN_CURRENT_OFFSET;
 
	return value;
}

static int sgm41513d_conver_to_charge_current_reg_val(int value)
{
	u8 reg = 0;

	/*
	 * For sgm41513d, REG02 The steps for setting the
	 * current in different current ranges are different,
	 * and the sizes of the ranges are also different.
	 * first interval less than 40ma, step 5ma, Interval Size 8,
	 * second         less than 130ma, step 10ma, Interval Size 8,
	 * third          less than 300ma, step 20ma, Interval Size 16,
	 * fourth         less than 540ma, step 30ma, Interval Size 24,
	 * fifth          less than 1500ma, step 60ma, Interval Size 32,
	 * sixth          less or equal to 3000ma, step 120ma, Interval Size 48,
	 */
	if (value < 40)
		reg = value / 5;
	else if (value < 130)
		reg = (value - 40) / 10 + 8;
	else if (value < 300)
		reg = (value - 130) / 20 + 16;
	else if (value < 540)
		reg = (value - 300) / 30 + 24;
	else if (value < 1500)
		reg = (value - 540) / 60 + 32;
	else if (value <= 3000)
		reg = (value - 1500) / 120 + 48;
	else
		hwlog_info("sgm41513d convert charge current fail\n");

	return reg;
}

static int bq2560x_set_charge_current(int value)
{
	int val;
	int tbat = DEFAULT_NORMAL_TEMP;

	if (!g_bq2560x_dev) {
		hwlog_err("chip not init\n");
		return -EIO;
	}

	bat_temp_get_rt_temperature(BAT_TEMP_MIXED, &tbat);
	hwlog_info("set charge current = %d\n", value);

	/*
	 * For sgm41511 charge IC, when the temperature range is 0-5, 5-10 and
	 * 45-50, the charge current needs reduce 60mA
	 * the current must over 2*60mA
	 */
	if (g_sgm41511_exist && (value >= 2 * REG02_ICHG_LSB)) {
		if (((tbat >= 0) && (tbat < 10)) || ((tbat >= 45) && (tbat < 50))) {
			value -= REG02_ICHG_LSB;
			hwlog_info("set reduced charge current = %d\n", value);
		}
	}

	/*
	 * For sy6974b charge IC, when the ICHG less than 840mA,
	 * the charge current needs write 840mA to make sure the chip in
	 * a full on mode, prevent smpl
	 */
	if ((g_bq2560x_dev->vendor_id == SY6974B_VENDOR_ID) && (value < SY6974B_ICHG_MIN)) {
		value = SY6974B_ICHG_MIN;
		hwlog_info("the chip need in full on mode, set charge current = %d\n", value);
	}

	if (g_bq2560x_dev->vendor_id == AH0072_VENDOR_ID)
		val = sgm41513d_conver_to_charge_current_reg_val(value);
	else
		val = (value - REG02_ICHG_BASE) / REG02_ICHG_LSB;

	hwlog_info("set_charge_current [%x]=0x%x, value=%d\n", BQ2560X_REG_CCC, val, value);

	return bq2560x_write_mask(BQ2560X_REG_CCC,
		BQ2560X_REG_CCC_ICHG_MASK,
		BQ2560X_REG_CCC_ICHG_SHIFT,
		val);
}

static int bq2560x_set_boost_current(int curr)
{
	int val;

	if (curr == BOOST_LIM_0P5A)
		val = REG02_BOOST_LIM_0P5A;
	else
		val = REG02_BOOST_LIM_1P2A;

	hwlog_info("set_boost_current [%x]=0x%x\n", BQ2560X_REG_CCC, val);

	return bq2560x_write_mask(BQ2560X_REG_CCC,
		BQ2560X_REG_CCC_BOOST_LIM_MASK,
		BQ2560X_REG_CCC_BOOST_LIM_SHIFT,
		val);
}

static int sc18601_set_cv(int value)
{
	int ret;
	int offset;
	int val;
	int mod;
	int ft_val;

	offset = value - REG04_VREG_BASE_SC18601;
	val = offset / REG04_VREG_LSB;
	mod = offset % REG04_VREG_LSB;
	ft_val = mod / SC18601_REG_CTL4_VBAT_REG_FT_STEP;

	hwlog_info("sc18601_set_cv [%x]=0x%x, ft_val=%d, value=%d\n",
		BQ2560X_REG_CVC, val, ft_val, value);

	ret = bq2560x_write_mask(BQ2560X_REG_CVC, BQ2560X_REG_CVC_VREG_MASK,
		BQ2560X_REG_CVC_VREG_SHIFT, val);
	ret += bq2560x_write_mask(SC18601_REG_CTL4, SC18601_REG_CTL4_VBAT_REG_FT_MASK,
		SC18601_REG_CTL4_VBAT_REG_FT_SHIFT, ft_val);
	if (ret)
		return -EIO;

	return 0;
}

static int sy6974k_set_cv(int value)
{
	int base_volt, base_step, base_val, val;

	value = (value < REG04_VREG_BASE0_SY6974K) ? REG04_VREG_BASE0_SY6974K : value;
	value = (value > REG04_VREG_MAX_VOLT_SY6974K) ? REG04_VREG_MAX_VOLT_SY6974K : value;

	base_volt = (value >= REG04_VREG_BASE1_SY6974K) ?
		REG04_VREG_BASE1_SY6974K : REG04_VREG_BASE0_SY6974K;
	base_step = (value >= REG04_VREG_BASE1_SY6974K) ?
		REG04_VREG_LSB1_SY6974K : REG04_VREG_LSB0_SY6974K;
	base_val = (value >= REG04_VREG_BASE1_SY6974K) ?
		REG04_VREG_BASE1_REG_SY6974K : 0;

	val = (value - base_volt) / base_step + base_val;
	hwlog_info("sy6974k_set_cv [%x]=0x%x, base_volt=%d, base_step=%d, base_val=%x, value=%d\n",
		BQ2560X_REG_CVC, val, base_volt, base_step, base_val, value);

	return bq2560x_write_mask(BQ2560X_REG_CVC,
		BQ2560X_REG_CVC_VREG_MASK_SY6974K,
		BQ2560X_REG_CVC_VREG_SHIFT_SY6974K,
		val);
}

static int ah0072_cal_cv(int value, int *val, int *ft_val)
{
	int offset;
	int delta_val;

	if (!val || !ft_val)
		return -EPERM;

	offset = value - REG04_VREG_BASE_AH0072;
	*val = offset / REG04_VREG_LSB;
	delta_val = (offset % REG04_VREG_LSB) / AH0072_REG_CTL4_VBAT_REG_FT_STEP;

	switch (delta_val) {
	case AH0072_REG_CTL4_VBAT_REG_FT_ADD8_VAL:
		*ft_val = AH0072_REG_CTL4_VBAT_REG_FT_ADD8;
		break;
	case AH0072_REG_CTL4_VBAT_REG_FT_MINUS8_VAL:
		*ft_val = AH0072_REG_CTL4_VBAT_REG_FT_MINUS8;
		*val += 1;
		break;
	case AH0072_REG_CTL4_VBAT_REG_FT_MINUS16_VAL:
		*ft_val = AH0072_REG_CTL4_VBAT_REG_FT_MINUS16;
		*val += 1;
		break;
	case AH0072_REG_CTL4_VBAT_REG_FT_0_VAL:
	default:
		*ft_val = AH0072_REG_CTL4_VBAT_REG_FT_0;
		break;
	}

	return 0;
}

static int ah0072_set_cv(int value)
{
	int ret;
	int val = 0;
	int ft_val = 0;

	if (ah0072_cal_cv(value, &val, &ft_val))
		return -EPERM;

	hwlog_info("ah0072_set_cv [%x]=0x%x, ft_val=%x, value=%d\n",
		BQ2560X_REG_CVC, val, ft_val, value);

	ret = bq2560x_write_mask(BQ2560X_REG_CVC, BQ2560X_REG_CVC_VREG_MASK,
		BQ2560X_REG_CVC_VREG_SHIFT, val);
	ret += bq2560x_write_mask(AH0072_REG_CTL7, AH0072_REG_CTL4_VBAT_REG_FT_MASK,
		AH0072_REG_CTL4_VBAT_REG_FT_SHIFT, ft_val);

	return ret;
}

static int bq2560x_reused_set_cv(int value)
{
	int val;
	struct bq2560x_device_info *di = g_bq2560x_dev;

	if (!di)
		return -EPERM;

	if (di->vendor_id == SC18601_VENDOR_ID)
		return sc18601_set_cv(value);
	
	if (di->vendor_id == SY6974B_VENDOR_ID && di->use_sy_new_cv)
		return sy6974k_set_cv(value);

	if (di->vendor_id == AH0072_VENDOR_ID)
		return ah0072_set_cv(value);

	val = (value - REG04_VREG_BASE) / REG04_VREG_LSB;

	hwlog_info("reused_set_cv [%x]=0x%x, value=%d\n", BQ2560X_REG_CVC, val, value);

	return bq2560x_write_mask(BQ2560X_REG_CVC,
		BQ2560X_REG_CVC_VREG_MASK,
		BQ2560X_REG_CVC_VREG_SHIFT,
		val);
}

static int bq2560x_set_terminal_voltage(int value)
{
	if ((g_bq2560x_cv > MIN_CV) && (value > g_bq2560x_cv)) {
		hwlog_info("set cv to custom_cv=%d\n", g_bq2560x_cv);
		value = g_bq2560x_cv;
	}

	g_cv_policy = value;

	if (g_cv_flag == 0)
		return bq2560x_reused_set_cv(value);
	else
		return 0;
}

static int bq2560x_set_dpm_voltage(int value)
{
	int val;

	if (value > REG06_VINDPM_MAX)
		value = REG06_VINDPM_MAX;
	else if (value < REG06_VINDPM_BASE)
		value = REG06_VINDPM_BASE;

	val = (value - REG06_VINDPM_BASE) / REG06_VINDPM_LSB;

	hwlog_info("set_dpm_voltage [%x]=0x%x\n", BQ2560X_REG_REG_BVTRC, val);

	return bq2560x_write_mask(BQ2560X_REG_REG_BVTRC,
		BQ2560X_REG_REG_BVTRC_VINDPM_MASK,
		BQ2560X_REG_REG_BVTRC_VINDPM_SHIFT,
		val);
}

static int sgm41513d_convert_to_iterm_reg_val(int value)
{
	u8 reg = 0;

	/*
	 * for sgm41513d, The method of setting the terminal current(reg02)
	 * is different from that of setting the current for bq2560x,
	 * as shown in the following table.
	 *      iterm[3:0]  iterm(mA)  iterm[3:0]  iterm(mA)
	 *      0000        5          1000        80
	 *      0001        10         1001        100
	 *      0010        15         1010        120
	 *      0011        20         1011        140
	 *      0100        30         1100        160
	 *      0101        40         1101        180
	 *      0110        50         1110        200
	 *      0111        60         1111        240
	 */
	if (value < 5)
		hwlog_info("sgm41513d iterm too low");
	else if (value < 20)
		reg = value / 5 - 1;
	else if (value <= 60)
		reg = (value / 10) + 1;
	else if (value < 240)
		reg = (value - 80) / 20 + 8;
	else
		reg = 15;

	return reg;
}

static int bq2560x_reused_set_iterm(int value)
{
	int val;

	if (!g_bq2560x_dev) {
		hwlog_err("chip not init\n");
		return -EIO;
	}

	if (g_bq2560x_dev->vendor_id == AH0072_VENDOR_ID)
		val = sgm41513d_convert_to_iterm_reg_val(value);
	else
		val = value < REG03_ITERM_BASE ? 0 : power_ceil(value - REG03_ITERM_BASE, REG03_ITERM_BASE);

	hwlog_info("reused_set_iterm [%x]=0x%x\n", BQ2560X_REG_PCTCC, val);

	return bq2560x_write_mask(BQ2560X_REG_PCTCC,
		BQ2560X_REG_PCTCC_ITERM_MASK,
		BQ2560X_REG_PCTCC_ITERM_SHIFT,
		val);
}

static int bq2560x_set_terminal_current(int value)
{
	int ret;
	int ichg = -power_platform_get_battery_current();
	struct bq2560x_device_info *di = g_bq2560x_dev;

	if (!di) {
		hwlog_err("di is null\n");
		return -1;
	}

	if ((ichg > LIMIT_CURR) && (di->ffc_support == 1)) {
		ret = bq2560x_reused_set_cv(g_cv_policy + REG04_VREG_LSB);
		if (ret) {
			hwlog_err("set high v_term fail, reset to g_cv_policy\n");
			ret = bq2560x_reused_set_cv(g_cv_policy);
			if (ret)
				hwlog_err("set g_cv_policy error\n");

			return bq2560x_reused_set_iterm(value);
		}

		g_cv_flag = 1;
		return bq2560x_reused_set_iterm(TERM_CURR);
	}

	ret = bq2560x_reused_set_iterm(value);
	if (ret) {
		hwlog_err("set the iterm fail\n");
		return ret;
	}

	g_cv_flag = 0;
	return bq2560x_reused_set_cv(g_cv_policy);
}

static int bq2560x_set_charge_enable(int enable)
{
	return bq2560x_write_mask(BQ2560X_REG_POC,
		BQ2560X_REG_POC_CHG_CONFIG_MASK,
		BQ2560X_REG_POC_CHG_CONFIG_SHIFT,
		enable);
}

static int sc18601_set_boost_voltage(int voltage)
{
	int ret;
	int val;

	/* BOOSTV = Offset + 100mV & BOOSTV[0:3] */
	val = (voltage - SC18601_REG_CTL4_BOOSTV_OFFSET) / SC18601_REG_CTL4_BOOSTV_STEP;
	hwlog_info("sc18601_set_boost_voltage [%x]=%d\n", BQ2560X_REG_CVC, val);
	/* BOOSTV[1]&[2]: reg0x6 bit4 and bit5 */
	ret = bq2560x_write_mask(BQ2560X_REG_REG_BVTRC, BQ2560X_REG_REG_BVTRC_BOOSTV_MASK,
	SC18601_REG_BOOSTV_SHIFT, val);
	/* BOOSTV[0]&[3]: reg0xd bit1 and bit2 */
	val = val / 4 + val % 2;
	ret += bq2560x_write_mask(SC18601_REG_CTL4, SC18601_REG_CTL4_BOOSTV_0_3_MASK,
		SC18601_REG_CTL4_BOOSTV_0_3_SHIFT, val);
	if (ret)
		return -EIO;

	return 0;
}

static int bq2560x_set_boost_voltage(int voltage)
{
	int val;

	if (g_bq2560x_dev->vendor_id == SC18601_VENDOR_ID)
		return sc18601_set_boost_voltage(voltage);

	if (voltage == BOOSTV_4850)
		val = REG06_BOOSTV_4P85V;
	else if (voltage == BOOSTV_5150)
		val = REG06_BOOSTV_5P15V;
	else if (voltage == BOOSTV_5300)
		val = REG06_BOOSTV_5P3V;
	else
		val = REG06_BOOSTV_5V;

	hwlog_info("set_boost_voltage [%x]=0x%x\n", BQ2560X_REG_REG_BVTRC, val);

	return bq2560x_write_mask(BQ2560X_REG_REG_BVTRC,
		BQ2560X_REG_REG_BVTRC_BOOSTV_MASK,
		BQ2560X_REG_REG_BVTRC_BOOSTV_SHIFT,
		val);
}

static int bq2560x_set_otg_enable(int enable)
{
	struct bq2560x_device_info *di = g_bq2560x_dev;

	if (!di) {
		hwlog_err("di is null\n");
		return -1;
	}

	/*
	 * notice:
	 * why enable irq when entry to OTG mode only?
	 * because we care VBUS overloaded OCP or OVP's interrupt in boost mode
	 */
	if (!di->irq_active && enable) {
		di->irq_active = 1; /* active */
		enable_irq(di->irq_int);
	} else if (di->irq_active && !enable) {
		di->irq_active = 0; /* inactive */
		disable_irq(di->irq_int);
	}

	return bq2560x_write_mask(BQ2560X_REG_POC,
		BQ2560X_REG_POC_OTG_CONFIG_MASK,
		BQ2560X_REG_POC_OTG_CONFIG_SHIFT,
		enable);
}

static int bq2560x_get_charge_state(unsigned int *state)
{
	u8 reg = 0;
	int ret;

	if (!state)
		return -1;

	if (!g_bq2560x_dev) {
		hwlog_err("di is null\n");
		return -1;
	}

	ret = bq2560x_read_byte(BQ2560X_REG_SS, &reg);
	if (ret < 0) {
		hwlog_err("get_charge_state:reg_ss fail\n");
		return -1;
	}

	hwlog_info("get_charge_state [%x]=0x%x\n", BQ2560X_REG_SS, reg);

	if ((reg & BQ2560X_REG_SS_CHRG_STAT_MASK) ==
		BQ2560X_REG_SS_CHRG_STAT_MASK)
		*state |= CHAGRE_STATE_CHRG_DONE;

	if (g_bq2560x_dev->vendor_id == SY6974B_VENDOR_ID) {
		ret = bq2560x_read_byte(BQ2560X_REG_VINS, &reg);
		if (ret < 0) {
			 hwlog_err("get_charge_state:reg_vins fail\n");
			 return -1;
		}
		hwlog_info("get_charge_state [%x]=0x%x\n", BQ2560X_REG_VINS, reg);
		if (!(reg & BQ2560X_REG_VINS_VBUS_GD_MASK))
			*state |= CHAGRE_STATE_NOT_PG;
	} else {
		if (!(reg & BQ2560X_REG_SS_PG_STAT_MASK))
			*state |= CHAGRE_STATE_NOT_PG;
	}

	ret = bq2560x_read_byte(BQ2560X_REG_F, &reg);
	if (ret < 0) {
		hwlog_err("get_charge_state:reg_f fail\n");
		return -1;
	}

	hwlog_info("get_charge_state [%x]=0x%x\n", BQ2560X_REG_F, reg);

	if ((reg & BQ2560X_REG_F_FAULT_WDT_MASK) ==
		BQ2560X_REG_F_FAULT_WDT_MASK)
		*state |= CHAGRE_STATE_WDT_FAULT;

	if ((reg & BQ2560X_REG_F_FAULT_BOOST_MASK) ==
		BQ2560X_REG_F_FAULT_BOOST_MASK)
		*state |= CHAGRE_STATE_VBUS_OVP;

	if ((reg & BQ2560X_REG_F_FAULT_BAT_MASK) ==
		BQ2560X_REG_F_FAULT_BAT_MASK)
		*state |= CHAGRE_STATE_BATT_OVP;

	return ret;
}

static int bq2560x_set_term_enable(int enable)
{
	int ret;
	unsigned int state = CHAGRE_STATE_NORMAL;

	ret = bq2560x_get_charge_state(&state);
	if (ret < 0) {
		hwlog_err("get_charge_state fail!!ret = %d\n", ret);
		return -EINVAL;
	}
	if (state & CHAGRE_STATE_CHRG_DONE) {
		hwlog_info("state is charge done, can not set term_enable\n");
		return -EINVAL;
	}

	hwlog_info("%s enable = %d\n", __func__, enable);

	return bq2560x_write_mask(BQ2560X_REG_CTTC,
		BQ2560X_REG_CTTC_EN_TERM_MASK,
		BQ2560X_REG_CTTC_EN_TERM_SHIFT,
		enable);
}

static int bq2560x_get_vbus_mv(unsigned int *vbus_mv)
{
	u8 otg_en = 0;
	int ret;
	u32 state = 0;
	int dc_vbus = 0;

	if (!g_bq2560x_dev)
		return -1;

	(void)bq2560x_read_mask(BQ2560X_REG_POC, BQ2560X_REG_POC_OTG_CONFIG_MASK,
		BQ2560X_REG_POC_OTG_CONFIG_SHIFT, &otg_en);
	if (otg_en) {
		*vbus_mv = 5000; /* 5000: fake voltage value for CTS */
		return 0;
	}

	/* this chip do not support get-vbus function, use dc chip vbus instead */
	if (g_bq2560x_dev->use_dc_vbus) {
		ret = dcm_get_ic_vbus(SC_MODE, CHARGE_IC_MAIN, &dc_vbus);
		*vbus_mv = dc_vbus > 0 ? dc_vbus : 0;
		return ret;
	}

	ret = bq2560x_get_charge_state(&state);
	if (ret) {
		hwlog_err("get_charge_state fail,ret:%d\n", ret);
		*vbus_mv = 0; /* fake voltage value for CTS */
		return -1;
	}
	if (CHAGRE_STATE_NOT_PG & state) {
		hwlog_info("get_charge_state,state:%u\n", state);
		*vbus_mv = 0; /* fake voltage value for CTS */
		return 0;
	}
	/*
	 * this chip do not support get-vbus function,
	 * for CTS, we return a fake vulue
	 */

	if (g_bq2560x_dev->vbus == ADAPTER_5V)
		*vbus_mv = 5000; /* 5000: fake voltage value for CTS */
	else if (g_bq2560x_dev->vbus == ADAPTER_9V)
		*vbus_mv = 9000; /* 9000: fake voltage value for CTS */

	return ret;
}

static void sgm41511h_set_safe_time_enable(bool enable)
{
	int ret;
	int retry;

	for (retry = 0; retry < RETRY_TIMES; retry++) {
		ret = bq2560x_write_mask(BQ2560X_REG_CTTC,
			BQ2560X_REG_CTTC_EN_TIMER_MASK,
			BQ2560X_REG_CTTC_EN_TIMER_SHIFT, enable);
		if (ret == 0)
			break;
	}
	if (ret != 0)
		hwlog_err("set safe time enable fail\n");

	hwlog_info("set safe time en = %d, retry = %d\n", enable, retry);
}

static void sgm41511h_reset_for_safe_time(void)
{
	if (g_safe_time_cnt < ENABLE_TIMES) {
		if ((power_get_monotonic_boottime() - g_boot_time) > g_safe_pre_time) {
			g_safe_time_cnt++;
			g_safe_pre_time += SAFE_TIME_RESET;
			sgm41511h_set_safe_time_enable(false);
			msleep(5); /* delay 5ms for set reg */
			sgm41511h_set_safe_time_enable(true);
			hwlog_info("safety timer,cnt=%d, safe_time=%d\n",
				g_safe_time_cnt, g_safe_pre_time);
		}
	}
}

static int bq2560x_reset_watchdog_timer(void)
{
	if (g_sgm41511_exist)
		sgm41511h_reset_for_safe_time();

	return bq2560x_write_mask(BQ2560X_REG_POC,
		BQ2560X_REG_POC_WDT_RESET_MASK,
		BQ2560X_REG_POC_WDT_RESET_SHIFT,
		0x01);
}

static int bq2560x_get_ilim(void)
{
	return 0;
}

static int bq2560x_check_charger_plugged(void)
{
	u8 reg = 0;
	int ret;

	ret = bq2560x_read_mask(BQ2560X_REG_SS,
		BQ2560X_REG_SS_PG_STAT_MASK,
		BQ2560X_REG_SS_PG_STAT_SHIFT,
		&reg);
	if (ret < 0)
		return 0;

	hwlog_info("check_charger_plugged [%x]=0x%x\n", BQ2560X_REG_SS, reg);

	if (reg == BQ2560X_REG_SS_VBUS_PLUGGED)
		return REG08_POWER_GOOD;

	return 0;
}

static int bq2560x_check_input_dpm_state(void)
{
	u8 reg = 0;
	int ret;

	ret = bq2560x_read_byte(BQ2560X_REG_VINS, &reg);
	if (ret < 0)
		return FALSE;

	hwlog_info("check_input_dpm_state [%x]=0x%x\n", BQ2560X_REG_VINS, reg);

	if (reg & BQ2560X_REG_VINS_VINDPM_STAT_MASK)
		return TRUE;

	return FALSE;
}

static int bq2560x_check_input_vdpm_state(void)
{
	u8 reg = 0;
	int ret;

	ret = bq2560x_read_byte(BQ2560X_REG_VINS, &reg);
	if (ret < 0)
		return FALSE;

	hwlog_info("check_input_vdpm_state [%x]=0x%x\n", BQ2560X_REG_VINS, reg);

	if (reg & BQ2560X_REG_VINS_VINDPM_STAT_MASK)
		return TRUE;

	return FALSE;
}

static int bq2560x_check_input_idpm_state(void)
{
	u8 reg = 0;
	int ret;

	ret = bq2560x_read_byte(BQ2560X_REG_VINS, &reg);
	if (ret < 0)
		return FALSE;

	hwlog_info("check_input_idpm_state [%x]=0x%x\n", BQ2560X_REG_VINS, reg);

	if (reg & BQ2560X_REG_VINS_IINDPM_STAT_MASK)
		return TRUE;

	return FALSE;
}

static int bq2560x_dump_register(char *reg_value, int size, void *dev_data)
{
	u8 reg[SC18601_REG_NUM] = {0};
	char buff[BUF_LEN] = {0};
	int reg_num = BQ2560X_REG_NUM;
	int i;
	int ret;

	if (!reg_value)
		return 0;

	memset(reg_value, 0, size);

	if (g_bq2560x_dev->vendor_id == SC18601_VENDOR_ID)
		reg_num = SC18601_REG_NUM;

	if (g_bq2560x_dev->vendor_id == AH0072_VENDOR_ID)
		reg_num = AH0072_REG_NUM;

	for (i = 0; i < reg_num; i++) {
		ret = bq2560x_read_byte(i, &reg[i]);
		if (ret)
			hwlog_err("dump_register read fail\n");

		snprintf(buff, BUF_LEN, "0x%-8.2x", reg[i]);
		strncat(reg_value, buff, strlen(buff));
	}

	return 0;
}

static int bq2560x_get_register_head(char *reg_head, int size, void *dev_data)
{
	char buff[BUF_LEN] = {0};
	int reg_num = BQ2560X_REG_NUM;
	int i;

	if (!reg_head)
		return 0;

	memset(reg_head, 0, size);

	if (g_bq2560x_dev->vendor_id == SC18601_VENDOR_ID)
		reg_num = SC18601_REG_NUM;

	if (g_bq2560x_dev->vendor_id == AH0072_VENDOR_ID)
		reg_num = AH0072_REG_NUM;

	for (i = 0; i < reg_num; i++) {
		snprintf(buff, BUF_LEN, "Reg[0x%-2.2x] ", i);
		strncat(reg_head, buff, strlen(buff));
	}

	return 0;
}

static int bq2560x_set_batfet_disable(int disable)
{
	hwlog_err("%s disable=%d\n", __func__, disable);
	return bq2560x_write_mask(BQ2560X_REG_MOC,
		BQ2560X_REG_MOC_BATFET_DISABLE_MASK,
		BQ2560X_REG_MOC_BATFET_DISABLE_SHIFT,
		disable);
}

static int bq2560x_set_batfet_reset_disable(int disable)
{
	hwlog_err("%s disable=%d\n", __func__, disable);
	return bq2560x_write_mask(BQ2560X_REG_MOC,
		BQ2560X_REG_MOC_BATFET_RST_EN_MASK,
		BQ2560X_REG_MOC_BATFET_RST_EN_SHIFT,
		disable);
}

static int bq2560x_set_batfet_disable_delay(int delay)
{
	if (delay > 0)
		delay = REG07_BATFET_DLY_10S;
	else
		delay = REG07_BATFET_DLY_0S;

	hwlog_err("%s delay=%d\n", __func__, delay);
	return bq2560x_write_mask(BQ2560X_REG_MOC,
		BQ2560X_REG_MOC_BATFET_DLY_MASK,
		BQ2560X_REG_MOC_BATFET_DLY_SHIFT,
		delay);
}

static int bq2560x_set_ship_mode_enable(bool enable)
{
	return bq2560x_set_batfet_disable(enable);
}

static int bq2560x_set_watchdog_timer(int value)
{
	int ret;
	int val;

	if ((value == WDT_BASE) || (g_hiz_mode == TRUE))
		val = REG05_WDT_DISABLE;
	else if (value <= WDT_40S)
		val = REG05_WDT_40S;
	else if (value <= WDT_80S)
		val = REG05_WDT_80S;
	else
		val = REG05_WDT_160S;

	ret = bq2560x_write_mask(BQ2560X_REG_CTTC,
		BQ2560X_REG_CTTC_WDT_MASK,
		BQ2560X_REG_CTTC_WDT_SHIFT,
		val);
	if (ret)
		hwlog_err("set_watchdog_timer write fail\n");

	hwlog_info("set_watchdog_timer [%x]=0x%x\n", BQ2560X_REG_CTTC, val);

	if (value > 0)
		ret = bq2560x_reset_watchdog_timer();

	return ret;
}

static int bq2560x_stop_charge_config(void)
{
	return bq2560x_set_watchdog_timer(CHAGRE_WDT_DISABLE);
}

static int bq2560x_set_charger_hiz(int enable)
{
	int ret;
	int ret2;
	static int first_in = 1;
	struct bq2560x_device_info *di = g_bq2560x_dev;

	if (!di) {
		hwlog_err("di is null\n");
		return 0;
	}

	if (g_shutdown_flag)
		enable = 0;

	if (enable > 0) {
#ifdef CONFIG_HUAWEI_USB_SHORT_CIRCUIT_PROTECT
		if ((di->hiz_iin_limit == 1) &&
			uscp_is_in_hiz_mode() && !uscp_is_in_rt_protect_mode()) {
			hiz_iin_limit_flag = HIZ_IIN_FLAG_TRUE;

			if (first_in) {
				hwlog_info("uscp_hiz_mode HIZ, enable:%d, set 100mA\n",
					enable);
				first_in = 0;
				/* set inputcurrent to 100mA */
				return bq2560x_set_input_current(IINLIM_100);
			} else {
				return 0;
			}
		} else {
#endif /* CONFIG_HUAWEI_USB_SHORT_CIRCUIT_PROTECT */
			ret = bq2560x_write_mask(BQ2560X_REG_ISC,
				BQ2560X_REG_ISC_EN_HIZ_MASK,
				BQ2560X_REG_ISC_EN_HIZ_SHIFT,
				TRUE);

			g_hiz_mode = TRUE;

			ret2 = bq2560x_set_watchdog_timer(CHAGRE_WDT_DISABLE);
			if (ret2)
				hwlog_err("set_charger_hiz write fail\n");
#ifdef CONFIG_HUAWEI_USB_SHORT_CIRCUIT_PROTECT
		}
#endif /* CONFIG_HUAWEI_USB_SHORT_CIRCUIT_PROTECT */
	} else {
		hiz_iin_limit_flag = HIZ_IIN_FLAG_FALSE;
		first_in = 1;

		ret = bq2560x_write_mask(BQ2560X_REG_ISC,
			BQ2560X_REG_ISC_EN_HIZ_MASK,
			BQ2560X_REG_ISC_EN_HIZ_SHIFT,
			FALSE);
		g_hiz_mode = FALSE;
	}

	return ret;
}

static struct charge_device_ops bq2560x_ops = {
	.chip_init = bq2560x_chip_init,
	.dev_check = bq2560x_device_check,
	.stop_charge_config = bq2560x_stop_charge_config,
	.set_input_current = bq2560x_set_input_current,
	.set_charge_current = bq2560x_set_charge_current,
	.set_terminal_voltage = bq2560x_set_terminal_voltage,
	.set_dpm_voltage = bq2560x_set_dpm_voltage,
	.set_terminal_current = bq2560x_set_terminal_current,
	.set_charge_enable = bq2560x_set_charge_enable,
	.set_otg_enable = bq2560x_set_otg_enable,
	.set_otg_current = bq2560x_set_boost_current,
	.set_term_enable = bq2560x_set_term_enable,
	.get_charge_state = bq2560x_get_charge_state,
	.reset_watchdog_timer = bq2560x_reset_watchdog_timer,
	.set_watchdog_timer = bq2560x_set_watchdog_timer,
	.set_batfet_disable = bq2560x_set_batfet_disable,
	.set_batfet_disable_delay = bq2560x_set_batfet_disable_delay,
	.get_ibus = bq2560x_get_ilim,
	.get_iin_set = bq2560x_get_input_current_set,
	.check_charger_plugged = bq2560x_check_charger_plugged,
	.check_input_dpm_state = bq2560x_check_input_dpm_state,
	.check_input_vdpm_state = bq2560x_check_input_vdpm_state,
	.check_input_idpm_state = bq2560x_check_input_idpm_state,
	.set_charger_hiz = bq2560x_set_charger_hiz,
	.get_charge_current = NULL,
	.set_boost_voltage = bq2560x_set_boost_voltage,
	.get_vbus = bq2560x_get_vbus_mv,
	.set_ship_mode_enable = bq2560x_set_ship_mode_enable,
};

static struct power_log_ops bq2560x_log_ops = {
	.dev_name = "bq2560x",
	.dump_log_head = bq2560x_get_register_head,
	.dump_log_content = bq2560x_dump_register,
};

static struct charger_otg_device_ops bq2560x_otg_ops = {
	.dev_name = "bq2560x",
	.otg_set_charger_enable = bq2560x_set_charge_enable,
	.otg_set_enable = bq2560x_set_otg_enable,
	.otg_set_current = bq2560x_set_boost_current,
	.otg_set_watchdog_timer = bq2560x_set_watchdog_timer,
	.otg_reset_watchdog_timer = bq2560x_reset_watchdog_timer,
};

static int bq2560x_batinfo_exit(void *dev_data)
{
	return 0;
}

static int bq2560x_batinfo_init(void *dev_data)
{
	return 0;
}

static int bq2560x_get_vbat_mv(void *dev_data)
{
	int batt_volt = 0;

	power_supply_get_int_prop(POWER_PLATFORM_BAT_PSY_NAME,
		POWER_SUPPLY_PROP_VOLTAGE_NOW, &batt_volt,
		POWER_SUPPLY_DEFAULT_VOLTAGE_NOW, POWER_UV_PER_MV);

	return batt_volt;
}

static int bq2560x_aux_sc_get_vbus_mv(int *vbus, void *dev_data)
{
	return bq2560x_get_vbus_mv(vbus);
}

static int bq2560x_get_ibat_ma(int *ibat, void *dev_data)
{
	return power_platform_get_battery_current();
}

static int bq2560x_get_ibus_ma(int *ibus, void *dev_data)
{
	int total_ibus = 0;
	int main_sc_ibus = 0;
	int ret;

	if (direct_charge_get_stage_status() != DC_STAGE_CHARGING)
		return 0;

	ret = adapter_get_output_current(ADAPTER_PROTOCOL_SCP, &total_ibus);
	ret += dcm_get_ic_ibus(SC_MODE, CHARGE_IC_MAIN, &main_sc_ibus);
	if (ret < 0)
		hwlog_err("%s error\n", __func__);

	*ibus = total_ibus - main_sc_ibus;
	hwlog_info("bq2560x_ibus=%d\n", *ibus);

	return 0;
}

static int bq2560x_get_device_temp(int *temp, void *dev_data)
{
	return DEFAULT_FAKE_DEVICE_TEMP;
}

static int bq2560x_get_vusb_mv(int *vusb, void *dev_data)
{
	return -EPERM;
}

static int bq2560x_get_vout_mv(int *vout, void *dev_data)
{
	return -EPERM;
}

static int bq2560x_sc_charge_init(void *dev_data)
{
	struct bq2560x_device_info *di = dev_data;

	if (!di) {
		hwlog_err("di is null\n");
		return -EINVAL;
	}

	hwlog_info("%s enter\n", __func__);
	return bq2560x_5v_chip_init(di);
}

static int bq2560x_sc_charge_exit(void *dev_data)
{
	return 0;
}

static int bq2560x_set_ovpthreshold(int threshld);

static int bq2560x_charge_enable(int enable, void *dev_data)
{
	struct bq2560x_device_info *di = dev_data;

	if (!di) {
		hwlog_err("di is null\n");
		return -EINVAL;
	}

	hwlog_info("%s enter, enable = %d\n", __func__, enable);
	if (enable == 0) {
		bq2560x_set_ovpthreshold(REG06_OVP_10P5V);
		return charge_set_hiz_enable(HIZ_MODE_ENABLE);
	}

	bq2560x_set_ovpthreshold(REG06_OVP_14P3V);
	charge_set_hiz_enable(HIZ_MODE_DISABLE);
	power_msleep(DT_MSLEEP_500MS, 0, NULL);

	bq2560x_set_input_current(di->sc_max_input_current);
	bq2560x_set_charge_current(di->sc_max_charge_current);
	bq2560x_set_charge_enable(enable);
	power_usleep(DT_USLEEP_10MS);

	return 0;
}

static int bq2560x_irq_enable(int enable, void *dev_data)
{
	return 0;
}

static int bq2560x_enable_adc(int enable, void *dev_data)
{
	return 0;
}

static int bq2560x_discharge(int enable, void *dev_data)
{
	return 0;
}

static int bq2560x_is_device_close(void *dev_data)
{
	return 0;
}

static int bq2560x_get_device_id(void *dev_data)
{
	struct bq2560x_device_info *di = dev_data;

	if (!di) {
		hwlog_err("di is null\n");
		return -EINVAL;
	}

	return di->device_id;
}

static int bq2560x_config_watchdog_ms(int time, void *dev_data)
{
	 return bq2560x_set_watchdog_timer(time);
}

static int bq2560x_kick_watchdog_ms(void *dev_data)
{
	return bq2560x_reset_watchdog_timer();
}

static struct dc_batinfo_ops bq2560x_batinfo_ops = {
	.init = bq2560x_batinfo_init,
	.exit = bq2560x_batinfo_exit,
	.get_bat_btb_voltage = bq2560x_get_vbat_mv,
	.get_bat_package_voltage = bq2560x_get_vbat_mv,
	.get_vbus_voltage = bq2560x_aux_sc_get_vbus_mv,
	.get_bat_current = bq2560x_get_ibat_ma,
	.get_ic_ibus = bq2560x_get_ibus_ma,
	.get_ic_temp = bq2560x_get_device_temp,
	.get_ic_vusb = bq2560x_get_vusb_mv,
	.get_ic_vout = bq2560x_get_vout_mv,
};

static struct dc_ic_ops bq2560x_sysinfo_ops = {
	.dev_name = "bq2560x",
	.ic_init = bq2560x_sc_charge_init,
	.ic_exit = bq2560x_sc_charge_exit,
	.ic_enable = bq2560x_charge_enable,
	.irq_enable = bq2560x_irq_enable,
	.ic_adc_enable = bq2560x_enable_adc,
	.ic_discharge = bq2560x_discharge,
	.is_ic_close = bq2560x_is_device_close,
	.get_ic_id = bq2560x_get_device_id,
	.config_ic_watchdog = bq2560x_config_watchdog_ms,
	.kick_ic_watchdog = bq2560x_kick_watchdog_ms,
};

static int bq2560x_fcp_init(struct bq2560x_device_info *info)
{
	struct device_node *np = NULL;

	if (!info || !info->dev || !info->dev->of_node) {
		hwlog_err("info is null\n");
		goto fail_init_fcp;
	}

	np = info->dev->of_node;
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"fcp_support", &(info->fcp_support), 0);

	if (!info->fcp_support)
		goto fail_init_fcp;

	/* gpio_fcp init to input mode */
	if (power_gpio_config_input(np,
		"gpio_fcp", "gpio_fcp", &info->gpio_fcp))
		goto fail_init_fcp;

	return 0;

fail_init_fcp:
	if (info)
		info->fcp_support = 0;
	return -1;
}

static int bq2560x_set_ovpthreshold(int threshld)
{
	int ret;
	int try_cnt = 0;

	while (try_cnt < RETRY_MAX3) {
		try_cnt++;
		ret = bq2560x_write_mask(BQ2560X_REG_REG_BVTRC,
			BQ2560X_REG_REG_BVTRC_OVP_MASK,
			BQ2560X_REG_REG_BVTRC_OVP_SHIFT,
			threshld);
		if (!ret) {
			hwlog_info("set ovp = %d success\n", threshld);
			return ret;
		}
		msleep(DELAY_10MS);
	}

	hwlog_err("set ovp %d fail\n", threshld);
	return -1;
}

static int bq2560x_is_ovpstate(bool *is_ovp)
{
	u8 reg = 0;
	int ret;

	ret = bq2560x_read_byte(BQ2560X_REG_VINS, &reg);
	if (ret) {
		*is_ovp = false;
		hwlog_err("get ovp state fail\n");
		return ret;
	}

	hwlog_info("fcp ovp read reg [%x]=0x%x\n", BQ2560X_REG_VINS, reg);
	if ((reg & BQ2560X_REG_VINS_ACOV_STAT_MASK) ==
		BQ2560X_REG_VINS_ACOV_STAT_MASK)
		*is_ovp = true;
	else
		*is_ovp = false;

	return ret;
}

static int bq2560x_fcp_reset(void *dev_data)
{
	struct bq2560x_device_info *di = g_bq2560x_dev;
	int ret;

	if (!di) {
		hwlog_err("di is null\n");
		return -EINVAL;
	}

	/* reset gpio to input status */
	gpio_set_value(di->gpio_fcp, GPIO_LOW);
	ret = gpio_direction_input(di->gpio_fcp);

	hwlog_info("fcp reset gpio_fcp\n");
	return ret;
}

static int bq2560x_fcp_adapter_detect(void *dev_data)
{
	int ret;
	int try_cnt = 0;
	bool is_ovp = false;
	struct bq2560x_device_info *di = g_bq2560x_dev;

	if (!di) {
		hwlog_err("di is null\n");
		return -1;
	}

	/* first: set ovp 6p2 to detect vbus 9v */
	ret = bq2560x_set_ovpthreshold(REG06_OVP_6P2V);
	if (ret)
		return -1;

	/*
	 * second:
	 * 1. wait 1.5s after DCP detect
	 * 2. set gpio_fcp to output high
	 * 3. wait 100ms up to 9v
	 */
	msleep(DELAY_1500MS);
	gpio_direction_output(di->gpio_fcp, GPIO_LOW);
	gpio_set_value(di->gpio_fcp, GPIO_HIGH);
	msleep(DELAY_100MS);
	hwlog_info("fcp detect set gpio_fcp = %d\n",
		gpio_get_value(di->gpio_fcp));

	/* third: detect ovp to judge 9v */
	while (try_cnt < RETRY_MAX3) {
		try_cnt++;
		ret = bq2560x_is_ovpstate(&is_ovp);
		if (!ret && (is_ovp == true)) {
			/* up 9v succ must reset ovp to 14p3 */
			(void)bq2560x_set_ovpthreshold(REG06_OVP_14P3V);
			hwlog_info("fcp detect upto9v succ\n");
			return 0;
		}
		msleep(DELAY_50MS);
	}

	/* fourth: reset when detect 9v failed */
	(void)bq2560x_set_ovpthreshold(REG06_OVP_14P3V);
	ret = bq2560x_fcp_reset(di);
	if (ret)
		hwlog_err("fcp reset fail\n");

	hwlog_err("fcp detect upto9v fail\n");
	return -1;
}

static int bq2560x_fcp_reg_read_block(int reg, int *val, int num,
	void *dev_data)
{
	int i;

	if (!val) {
		hwlog_err("val is null\n");
		return -1;
	}

	for (i = 0; i < num; i++)
		val[i] = 0;

	return 0;
}

static int bq2560x_fcp_reg_write_block(int reg, const int *val, int num,
	void *dev_data)
{
	return 0;
}

static struct hwfcp_ops bq2560x_fcp_protocol_ops = {
	.chip_name = "bq2560x",
	.reg_read = bq2560x_fcp_reg_read_block,
	.reg_write = bq2560x_fcp_reg_write_block,
	.detect_adapter = bq2560x_fcp_adapter_detect,
	.soft_reset_master = bq2560x_fcp_reset,
	.soft_reset_slave = bq2560x_fcp_reset,
	.get_master_status = NULL,
	.stop_charging_config = bq2560x_fcp_reset,
	.is_accp_charger_type = NULL,
};

static void bq2560x_monitor_work(struct work_struct *work)
{
	u8 value = 0;
	char buf[POWER_DSM_BUF_SIZE_0256] = { 0 };
	struct bq2560x_device_info *di = NULL;

	if (!work)
		return;

	hwlog_info("%s +++\n", __func__);
	di = container_of(work, struct bq2560x_device_info, monitor_work.work);
	if (!di || di->is_charging)
		return;

	bq2560x_read_byte(BQ2560X_REG_MOC, &value);
	hwlog_info("%s reg[0x07]=0x%x\n", __func__, value);
	if (value & BQ2560X_REG_MOC_BATFET_DISABLE_MASK) {
		snprintf_s(buf, sizeof(buf), sizeof(buf) - 1,
			"batfet shutdown happened, reg[0x07]=0x%x\n", value);
		power_dsm_report_dmd(POWER_DSM_BATTERY, ERROR_DISCHARGE_CURR_OVERHIGH, buf);
		bq2560x_write_mask(BQ2560X_REG_MOC, BQ2560X_REG_MOC_BATFET_DISABLE_MASK,
			BQ2560X_REG_MOC_BATFET_DISABLE_SHIFT, 0);
		bq2560x_read_byte(BQ2560X_REG_MOC, &value);
		hwlog_info("%s reg read again, reg[0x07]=0x%x\n", __func__, value);
	}

	hwlog_info("%s ---\n", __func__);
	queue_delayed_work(system_freezable_power_efficient_wq,
		&di->monitor_work, msecs_to_jiffies(MONITOR_WORK_DELAY_TIME));
}

static void bq2560x_irq_work(struct work_struct *work)
{
	struct bq2560x_device_info *di = NULL;
	u8 reg = 0;
	int ret;

	if (!work) {
		hwlog_err("work is null\n");
		return;
	}

	di = container_of(work, struct bq2560x_device_info, irq_work);
	if (!di) {
		hwlog_err("di is null\n");
		return;
	}

	msleep(100); /* sleep 100ms */

	ret = bq2560x_read_byte(BQ2560X_REG_F, &reg);
	if (ret)
		hwlog_err("irq_work read fail\n");

	hwlog_err("boost_ovp_reg [%x]=0x%x\n", BQ2560X_REG_F, reg);

	if (reg & BQ2560X_REG_F_FAULT_BOOST_MASK) {
		hwlog_info("boost ocp happened\n");
		power_event_anc_notify(POWER_ANT_CHARGE_FAULT,
			POWER_NE_CHG_FAULT_BOOST_OCP, NULL);
	}

	if (di->irq_active == 0) {
		di->irq_active = 1;
		enable_irq(di->irq_int);
	}
}

static irqreturn_t bq2560x_interrupt(int irq, void *_di)
{
	struct bq2560x_device_info *di = _di;

	if (!di) {
		hwlog_err("di is null\n");
		return IRQ_HANDLED;
	}

	hwlog_info("bq2560x int happened\n");

	if (di->irq_active == 1) {
		di->irq_active = 0;
		disable_irq_nosync(di->irq_int);
		schedule_work(&di->irq_work);
	} else {
		hwlog_info("the irq is not enable, do nothing\n");
	}

	return IRQ_HANDLED;
}

static void bq2560x_init_ops_dev_data(struct bq2560x_device_info *di)
{
	memcpy(&di->sc_ops, &bq2560x_sysinfo_ops, sizeof(struct dc_ic_ops));
	di->sc_ops.dev_data = (void *)di;
	memcpy(&di->batinfo_ops, &bq2560x_batinfo_ops, sizeof(struct dc_batinfo_ops));
	di->batinfo_ops.dev_data = (void *)di;

	if (!di->ic_role) {
		snprintf(di->name, CHIP_DEV_NAME_LEN, "bq2560x");
	} else {
		snprintf(di->name, CHIP_DEV_NAME_LEN, "bq2560x_%d", di->ic_role);
		di->sc_ops.dev_name = di->name;
	}
}

static void bq2560x_ops_register(struct bq2560x_device_info *di)
{
	int ret;

	bq2560x_init_ops_dev_data(di);

	ret = dc_ic_ops_register(SC_MODE, di->ic_role, &di->sc_ops);
	ret += dc_batinfo_ops_register(di->ic_role, &di->batinfo_ops, di->device_id);
	if (ret)
		hwlog_err("sc ops or sysinfo ops register fail\n");
}

static void bq2560x_parse_dts(struct device_node *np, struct bq2560x_device_info *di)
{
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"hiz_iin_limit", &(di->hiz_iin_limit), 0);
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"custom_cv", &g_bq2560x_cv, 0);
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"ffc_support", &di->ffc_support, 1);
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"ovp_th", &g_bq2560x_ovp, REG06_OVP_10P5V);
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"support_sc", &di->support_sc, 0);
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"ic_role", &di->ic_role, IC_TWO);
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"sc_max_input_current", &di->sc_max_input_current, DEFAULT_SC_MAX_INPUT_CURRENT);
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"sc_max_charge_current", &di->sc_max_charge_current, DEFAULT_SC_MAX_CHARGE_CURRENT);
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"init_clear_hiz", &di->init_clear_hiz, 0);
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"shipmode_clear_hiz", &di->shipmode_clear_hiz, 0);
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"use_dc_vbus", &di->use_dc_vbus, 0);
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"disable_batfet_reset", &di->disable_batfet_reset, 0);
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"monitor_batfet_en", &di->monitor_batfet_en, 0);
	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"use_sy_new_cv", &di->use_sy_new_cv, 0);
}

static void bq2560x_reg_init(struct bq2560x_device_info *di)
{
	int ret;

	(void)bq2560x_set_watchdog_timer(CHAGRE_WDT_DISABLE);

	if (di->init_clear_hiz && g_sgm41511_exist) {
		ret = bq2560x_set_charger_hiz(0);
		if (ret)
			hwlog_err("set bq2560x set hiz fail\n");
	}

	if (di->shipmode_clear_hiz) {
		ret = bq2560x_set_charger_hiz(0);
		if (ret)
			hwlog_err("shipmode set bq2560x set hiz fail\n");
	}

	if (di->disable_batfet_reset) {
		ret = bq2560x_set_batfet_reset_disable(REG07_BATFET_RST_DISABLE);
		if (ret)
			hwlog_err("disable batfet reset fail\n");
	}
}

static int bq2560x_event_notifier_call(struct notifier_block *nb,
	unsigned long event, void *data)
{
	struct bq2560x_device_info *di = g_bq2560x_dev;

	if (!di || !di->monitor_batfet_en)
		return NOTIFY_OK;

	switch (event) {
	case POWER_NE_CHARGING_START:
		di->is_charging = true;
		cancel_delayed_work(&di->monitor_work);
		break;
	case POWER_NE_CHARGING_STOP:
		di->is_charging = false;
		cancel_delayed_work(&di->monitor_work);
		queue_delayed_work(system_freezable_power_efficient_wq,
			&di->monitor_work, msecs_to_jiffies(MONITOR_WORK_DEBOUNCE_TIME));
		break;
	default:
		break;
	}

	return NOTIFY_OK;
}


static int bq2560x_probe(struct i2c_client *client,
	const struct i2c_device_id *id)
{
	int ret;
	struct bq2560x_device_info *di = NULL;
	struct device_node *np = NULL;
	struct power_devices_info_data *power_dev_info = NULL;

	if (g_bq2560x_dev && g_bq2560x_dev->probe_ok) {
		hwlog_err("bq2560x probe ok already\n");
		return -ENODEV;
	}

	if (!client || !client->dev.of_node || !id)
		return -ENODEV;

	di = kzalloc(sizeof(*di), GFP_KERNEL);
	if (!di)
		return -ENOMEM;

	g_bq2560x_dev = di;

	di->dev = &client->dev;
	np = di->dev->of_node;
	di->client = client;
	i2c_set_clientdata(client, di);
	mutex_init(&di->io_lock);

	INIT_WORK(&di->irq_work, bq2560x_irq_work);
	INIT_DELAYED_WORK(&di->monitor_work, bq2560x_monitor_work);

	(void)power_dts_read_u32(power_dts_tag(HWLOG_TAG), np,
		"need_check_sgm41511", &di->need_check_sgm41511, 1);
	hwlog_info("need_check_sgm41511 = %d\n", di->need_check_sgm41511);
	/* check if bq2560x exist */
	if (bq2560x_device_check() == CHARGE_IC_BAD) {
		hwlog_err("bq2560x not exists\n");
		ret = -EINVAL;
		goto bq2560x_fail_0;
	}

	if (di->need_check_sgm41511 && sgm41511_device_check())
		g_sgm41511_exist = 1;
	di->device_id = BUCK_BQ2560X;
	/* close otg */
	bq2560x_write_mask(BQ2560X_REG_POC, BQ2560X_REG_POC_OTG_CONFIG_MASK,
		BQ2560X_REG_POC_OTG_CONFIG_SHIFT, 0);

	bq2560x_parse_dts(np, di);
	/* set gpio to control CD pin to disable/enable bq2560x IC */
	ret = power_gpio_config_output(np,
		"gpio_cd", "charger_cd", &di->gpio_cd, 0);
	if (ret)
		goto bq2560x_fail_0;

	ret = power_gpio_config_interrupt(np,
		"gpio_int", "charger_int", &di->gpio_int, &di->irq_int);
	if (ret)
		goto bq2560x_fail_1;

	ret = request_irq(di->irq_int, bq2560x_interrupt, IRQF_TRIGGER_FALLING,
		"charger_int_irq", di);
	if (ret) {
		hwlog_err("gpio irq request fail\n");
		di->irq_int = -1;
		goto bq2560x_fail_2;
	}

	disable_irq(di->irq_int);
	di->irq_active = 0;

	ret = charge_ops_register(&bq2560x_ops, BUCK_IC_TYPE_THIRDPARTY);
	if (ret) {
		hwlog_err("bq2560x charge ops register fail\n");
		goto bq2560x_fail_3;
	}

	ret = charger_otg_ops_register(&bq2560x_otg_ops);
	if (ret) {
		hwlog_err("bq2560x charger_otg ops register fail\n");
		goto bq2560x_fail_3;
	}

	di->nb.notifier_call = bq2560x_event_notifier_call;
	ret = power_event_bnc_register(POWER_BNT_CHARGING, &di->nb);
	if (ret)
		hwlog_err("register POWER_BNT_CHARGING nb fail\n");

	/* set bq2560x boost voltage */
	ret = bq2560x_set_boost_voltage(BOOSTV_5000);
	if (ret < 0)
		hwlog_err("set bq2560x boost voltage fail\n");

	if (!bq2560x_fcp_init(di))
		hwfcp_ops_register(&bq2560x_fcp_protocol_ops);

	power_dev_info = power_devices_info_register();
	if (power_dev_info) {
		power_dev_info->dev_name = di->dev->driver->name;
		power_dev_info->dev_id = 0;
		power_dev_info->ver_id = 0;
	}

	bq2560x_log_ops.dev_data = (void *)di;
	power_log_ops_register(&bq2560x_log_ops);
	if (di->support_sc)
		bq2560x_ops_register(di);

	bq2560x_reg_init(di);
	if (di->monitor_batfet_en)
		queue_delayed_work(system_freezable_power_efficient_wq, &di->monitor_work,
			MONITOR_WORK_DELAY_TIME);
	hwlog_info("bq2560x success\n");
	g_bq2560x_dev->probe_ok = true;
	return 0;

bq2560x_fail_3:
	free_irq(di->irq_int, di);
bq2560x_fail_2:
	gpio_free(di->gpio_int);
bq2560x_fail_1:
	gpio_free(di->gpio_cd);
bq2560x_fail_0:
	mutex_destroy(&di->io_lock);
	kfree(di);
	g_bq2560x_dev = NULL;
	hwlog_err("bq2560x_failed\n");

	return ret;
}

static void bq2560x_shutdown(struct i2c_client *client)
{
	int ret;
	u8 part;
	u8 reg = 0;

	ret = bq2560x_read_byte(BQ2560X_REG_VPRS, &reg);
	if (ret)
		return;

	part = reg & BQ2560X_REG_VPRS_PART_MASK;
	if (part) {
		g_shutdown_flag = TRUE;
		/* 0:close hiz mode */
		ret = bq2560x_set_charger_hiz(0);
		if (ret)
			hwlog_err("set bq2560x set hiz fail\n");
	}
}

static int bq2560x_remove(struct i2c_client *client)
{
	struct bq2560x_device_info *di = i2c_get_clientdata(client);

	if (!di)
		return -ENODEV;

	power_event_bnc_unregister(POWER_BNT_CHARGING, &di->nb);
	gpio_set_value(di->gpio_cd, 1);

	if (di->gpio_cd)
		gpio_free(di->gpio_cd);

	if (di->irq_int)
		free_irq(di->irq_int, di);

	if (di->gpio_int)
		gpio_free(di->gpio_int);

	if (di->gpio_fcp)
		gpio_free(di->gpio_fcp);

	mutex_destroy(&di->io_lock);
	kfree(di);
	g_bq2560x_dev = NULL;

	return 0;
}

MODULE_DEVICE_TABLE(i2c, bq25601);
static const struct of_device_id bq2560x_of_match[] = {
	{
		.compatible = "huawei,bq2560x_charger",
		.data = NULL,
	},
	{
		.compatible = "huawei,sgm41513d_charger",
		.data = NULL,
	},
	{},
};

static const struct i2c_device_id bq2560x_i2c_id[] = {
	{ "bq2560x_charger", 0 }, { "sgm41513d_charger", 0 }, {}
};

static struct i2c_driver bq2560x_driver = {
	.probe = bq2560x_probe,
	.remove = bq2560x_remove,
	.shutdown = bq2560x_shutdown,
	.id_table = bq2560x_i2c_id,
	.driver = {
		.owner = THIS_MODULE,
		.name = "bq2560x_charger",
		.of_match_table = of_match_ptr(bq2560x_of_match),
	},
};

static int __init bq2560x_init(void)
{
	return i2c_add_driver(&bq2560x_driver);
}

static void __exit bq2560x_exit(void)
{
	i2c_del_driver(&bq2560x_driver);
}

module_init(bq2560x_init);
module_exit(bq2560x_exit);

MODULE_LICENSE("GPL v2");
MODULE_DESCRIPTION("bq2560x charger module driver");
MODULE_AUTHOR("Huawei Technologies Co., Ltd.");
